<?php

/**
 * Plugin Name: GM 2factor token authentication
 * Plugin URI: https://gianlucametello.it/2factor-token-authentication/
 * Description: Plugin di autenticazione 2 fattori implementato nelle pagine di login
 * Version: 1.0
 * Author: Gianluca Metello
 * Author URI: http://www.gianlucametello.it
 */

add_filter( 'authenticate', 'custom_authenticate_user', 21 );
$key_site = get_option("gm_2factor_key_token");

function gm_register_login_submenu_page() {        
        add_submenu_page(
                'gm-core', 
                'GM 2Factor Token', 
                'GM 2Factor Token',
                'manage_options', 
                'gm-2factor-token-authentication/gm-2factor-token-authentication-admin.php', 
                ''
                );
}
add_action( 'admin_menu', 'gm_register_login_submenu_page' , 70);

 // register jquery and style on initialization
add_action('init', 'register_script');
function register_script() {
    wp_register_script( 'gm_login_js', plugins_url('/js/gm-2factor-token-authentication.js', __FILE__), array('jquery'), '2.5.1' );
    wp_register_style( 'gm_login_css', plugins_url('/css/gm-2factor-token-authentication.css', __FILE__), false, '1.0.0', 'all');
    wp_localize_script( 'gm_login_js', 'ajax_send', array(
       'url'      => admin_url( 'admin-ajax.php' ),
       'security' => wp_create_nonce('ajax_send_nonce_string')
   ));
}

// use the registered jquery and style above
add_action('login_head', 'gm_enqueue_style');
add_action('woocommerce_before_customer_login_form', 'gm_enqueue_style');

function gm_enqueue_style(){
   wp_enqueue_script('gm_login_js');
   wp_enqueue_style( 'gm_login_css' );
}

function gm_admin_enqueue_style() {
wp_register_script( 'gm_admin_enqueue_style_js', plugin_dir_url( __FILE__ ).'js/gm-2factor-token-authentication-admin.js', array( 'jquery' ), '1.0', true );
    wp_enqueue_script( 'gm_admin_enqueue_style_js' );
    wp_localize_script( 'gm_admin_enqueue_style_js', 'ajax_send', array(
       'url'      => admin_url( 'admin-ajax.php' ),
       'security' => wp_create_nonce('ajax_send_nonce_string')
   ));
}
add_action( 'admin_enqueue_scripts', 'gm_admin_enqueue_style' );

function genera_2factor_token()
{
    global $key_site;
    if (!$key_site) return;
    check_ajax_referer( 'ajax_send_nonce_string', 'security' );
    
    // set post fields
            $post = [
                'key' => $key_site,
                'url' => 'https://gianlucametello.it/wp-content/plugins/gm-2factor/get-2factor-token.php',
                'siteback' => home_url()
            ];
    
    $response = gm_exec_curl($post);

    echo $response;
    die();
}
// Utenti autenticati
add_action( 'wp_ajax_nopriv_genera_2factor_token', 'genera_2factor_token' );
// Utenti non autenticati
add_action( 'wp_ajax_genera_2factor_token', 'genera_2factor_token' );


function custom_authenticate_user( $user, $registration=false ) {
    global $key_site;
    if (!$key_site) return $user ;
//    if (isset($_POST["wp-submit"])) {
            $fact1 = (isset($_POST["2factor_1"])?$_POST["2factor_1"]:"");
            $fact2 = (isset($_POST["2factor_2"])?$_POST["2factor_2"]:"");
            $fact3 = (isset($_POST["2factor_3"])?$_POST["2factor_3"]:"");
            $fact4 = (isset($_POST["2factor_4"])?$_POST["2factor_4"]:"");
            $fact5 = (isset($_POST["2factor_5"])?$_POST["2factor_5"]:"");
            $twofactor = $fact1.$fact2.$fact3.$fact4.$fact5;
            if ($fact1 == "" || $fact2 == "" || $fact3 == "" || $fact4 == "" || $fact5 =="") {
                if (isset($_POST["wp-submit"])) {
                    return new WP_Error( 'fallimento login', __( "Devi inserire un codice 2 fattori corretto", "gm-2factor" ) );
                } else {
                    if (!$_POST)
                        return $user;
                }
            }

            // set post fields
            $post = [
                'key' => $key_site,
                'twofactor' => $twofactor,
                'verify'   => 1,
                'url' => 'https://gianlucametello.it/authentication-2factor-api-return/',
                'siteback' => home_url()
            ];
            
            $response = gm_exec_curl($post);

            // do anything you want with your response
            if ($response == "success") {
                if ($registration) return true;
                return $user; // You should return the WP_User instance or a WP_Error instance on error.
            } else {
                if ($registration) return false;
                return new WP_Error( 'fallimento login', __( "Il codice non è stato autenticato", "gm-2factor" ) );
            }
//    }
    return $user;
    
}

add_action('login_form','my_added_login_field');
add_action('woocommerce_login_form','my_added_login_field');
add_action('woocommerce_register_form','my_added_login_field');

function my_added_login_field(){
    global $key_site;
    if (!$key_site) return;
?>
<div class="main_generation_token">
    <p class="genera_token_box">
        <button class="button-15 generate-token" role="button">Richiedi 2Factor Authentication</button></p>
        <!--<label for="gm_two_factor">Genera <a class="generate-token" href="#" target="_blank">GM 2Factor</a> per accedere</label></p>-->
    <div class="loader_captcha"><div class="loader"></div></div>
        <div class="result_captcha"></div>
            <div class="gm_login_form">
                <div><input class="captcha" type="text" name="2factor_1" placeholder="X" style="text-align:center" maxlength="1" tabindex="1" autocomplete="off"/></div>
                <div><input class="captcha" type="text" name="2factor_2" placeholder="X" style="text-align:center" maxlength="1" tabindex="2" autocomplete="off"/></div>
                <div><input class="captcha" type="text" name="2factor_3" placeholder="X" style="text-align:center" maxlength="1" tabindex="3" autocomplete="off"/></div>
                <div><input class="captcha" type="text" name="2factor_4" placeholder="X" style="text-align:center" maxlength="1" tabindex="4" autocomplete="off"/></div>
                <div><input class="captcha" type="text" name="2factor_5" placeholder="X" style="text-align:center" maxlength="1" tabindex="5" autocomplete="off"/></div>
            </div>
</div>
<?php
}

function gm_exec_curl($post) {
    $ch = curl_init($post['url']);
    unset($post['url']);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $post);

    // execute!
    $response = curl_exec($ch);

    // close the connection, release resources used
    curl_close($ch);
    
    return $response;
}

// WOOCOMMERCE SECTION
// Extra register fields validation
add_action( 'woocommerce_register_post', 'wc_validate_reg_form_fields', 10, 3 );
function wc_validate_reg_form_fields( $username, $email, $validation_errors ) {
            if (!custom_authenticate_user( null, true)) {
                $validation_errors->add( 'extra_fields', "codice a 2 fattori errato, riprova");
            }
            
    return $validation_errors;
}

add_action('lostpassword_post', 'my_validate_lost_pass_form');
function my_validate_lost_pass_form($errors)  {
     //$errors->add( 'my_error_name', "<strong>ERROR</strong>: Access denied." );
    return $errors;
}

// VISUALIZZARE A VIDEO IL 2FACTOR SU PASSWORD DIMENTICATA
add_filter( 'woocommerce_lost_password_message', 'wp_kama_woocommerce_lost_password_message_filter' );
function wp_kama_woocommerce_lost_password_message_filter( $esc_html__ ){
	return "";
}

add_action( 'wp_ajax_nopriv_reset_token', 'reset_token' );
add_action( 'wp_ajax_reset_token', 'reset_token' );

function reset_token() {
    delete_option( 'gm_2factor_key_token' );
}



